/*
 *  R : A Computer Language for Statistical Data Analysis
 *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
 *  Copyright (C) 1997--2005  The R Core Team
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, a copy is available at
 *  http://www.r-project.org/Licenses/
 */

/*
 * Modificaitons and Stubs
 */

#if defined(RMIN_SUBSTUFF) && ! defined(RMIN_ARRAY_BIND)
# define RMIN_ARRAY_BIND
#endif

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <time.h>
#include "Defn.h"
#include <Internal.h>
#include <R_ext/Random.h>
#include <Rmath.h>
#include <Rconnections.h>
#include <Startup.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>

#ifdef DODO
void Rvprintf(const char *format, va_list arg)
{
    va_list argcopy;
    va_copy(argcopy, arg);
    vfprintf(stdout, format, argcopy);
    va_end(argcopy);
    fflush(stdout);
}
#endif

double BM_norm_keep;
N01type N01_kind;
int R_OutputCon;

void F77_SYMBOL(intpr) (const char *a, int *b, int *c, int *d) {}

void set_rl_word_breaks(const char *str) {}

void GEonExit(void) {}

int findInterval(double *xt, int n, double x,
		 Rboolean rightmost_closed,  Rboolean all_inside, int ilo,
		 int *mflag)
{
    error("'findInterval' is not supported");
    return 0;
}

Rboolean switch_stdout(int icon, int closeOnExit) { return FALSE; }

#include <lzma.h>
uint64_t lzma_crc64(const uint8_t *buf, size_t size, uint64_t crc) { return 0; }

/* from datetime.c */
double currentTime(void)
{
    double ans = NA_REAL;

#ifdef HAVE_TIMESPEC_GET
    struct timespec tp;
    int res = timespec_get(&tp, TIME_UTC);
    if(res != 0)
	ans = (double) tp.tv_sec + 1e-9 * (double) tp.tv_nsec;
#elif defined(HAVE_CLOCK_GETTIME) && defined(CLOCK_REALTIME)
    struct timespec tp;
    int res = clock_gettime(CLOCK_REALTIME, &tp);
    if(res == 0)
	ans = (double) tp.tv_sec + 1e-9 * (double) tp.tv_nsec;

#elif defined(HAVE_GETTIMEOFDAY)
    /* Mac OS X, mingw.org */
    struct timeval tv;
    int res = gettimeofday(&tv, NULL);
    if(res == 0)
	ans = (double) tv.tv_sec + 1e-6 * (double) tv.tv_usec;

#else
    /* No known current OSes */
    time_t res = time(NULL);
    if(res != (time_t)(-1)) /* -1 must be an error as the real value -1
			       was ca 1969 */
	ans = (double) res;
#endif

#ifndef HAVE_POSIX_LEAPSECONDS
    /* No known current OSes */
    /* Disallowed by POSIX (1988-):
       http://www.mail-archive.com/leapsecs@rom.usno.navy.mil/msg00109.html
       http://en.wikipedia.org/wiki/Unix_time
    */
    if (!ISNAN(ans)) {
	ans -= n_leapseconds;
    }
#endif
    return ans;
}

/* from datetime.c */
unsigned int TimeToSeed(void)
{
    unsigned int seed, pid = getpid();
#if defined(HAVE_CLOCK_GETTIME) && defined(CLOCK_REALTIME)
    {
	struct timespec tp;
	clock_gettime(CLOCK_REALTIME, &tp);
	seed = (unsigned int)(((uint_least64_t) tp.tv_nsec << 16) ^ tp.tv_sec);
    }
#elif defined(HAVE_GETTIMEOFDAY)
    {
	struct timeval tv;
	gettimeofday (&tv, NULL);
	seed = (unsigned int)(((uint_least64_t) tv.tv_usec << 16) ^ tv.tv_sec);
    }
#else
    /* C89, so must work */
    seed = (Int32) time(NULL);
#endif
    seed ^= (pid <<16);
    return seed;
}

#define max2(x, y) ((x < y) ? y : x)
int imax2(int x, int y) { return max2(x, y); }
double fmax2(double x, double y) { return max2(x, y); }

#ifndef RMIN_SOURCE
void parseError(SEXP call, int linenum) { error("parse error"); }
#endif

/* from summary.c */
SEXP fixup_NaRm(SEXP args)
{
    SEXP t, na_value;

    /* Need to make sure na.rm is last and exists */
    na_value = ScalarLogical(FALSE);
    for(SEXP a = args, prev = R_NilValue; a != R_NilValue; a = CDR(a)) {
	if(TAG(a) == R_NaRmSymbol) {
	    if(CDR(a) == R_NilValue) return args;
	    na_value = CAR(a);
	    if(prev == R_NilValue) args = CDR(a);
	    else SETCDR(prev, CDR(a));
	}
	prev = a;
    }

    PROTECT(na_value);
    t = CONS(na_value, R_NilValue);
    UNPROTECT(1);
    PROTECT(t);
    SET_TAG(t, R_NaRmSymbol);
    if (args == R_NilValue)
	args = t;
    else {
	SEXP r = args;
	while (CDR(r) != R_NilValue) r = CDR(r);
	SETCDR(r, t);
    }
    UNPROTECT(1);
    return args;
}

int getActiveSink(int i) { return 0; }

struct Rconn term;

static int printer(struct Rconn *con, const char *fmt, va_list args)
{
    return vfprintf(stdout, fmt, args);
}
static int flusher(struct Rconn *con) { return fflush(stdout); }
Rconnection getConnection(int n)
{
    term.vfprintf = printer;
    term.fflush = flusher;
    return &term;
}
Rconnection getConnection_no_err(int n) { return NULL; }

void InitDynload(void) {}
void Init_R_Variables(SEXP env) {}
void *R_FindSymbol(const char *name) { return NULL; }

const char *locale2charset(const char *name) { return NULL; }

void process_site_Renviron(void) {}
void process_system_Renviron(void) {}
void process_user_Renviron(void) {}
void R_check_locale(void) {}
void R_CleanTempDir(void) {}

double sign(double x)
{
    if (ISNAN(x))
	return x;
    return ((x > 0) ? 1 : ((x == 0)? 0 : -1));
}

#include <R_ext/RStartup.h>
void R_common_command_line(int *pac, char **argv, Rstart Rp) {}

void Rf_InitConnections(void) {}
void Rf_InitEd(void) {}
void Rf_InitGraphics(void) {}
void Rf_PrintGreeting(void) {}

Rboolean R_current_trace_state(void) { return FALSE; }

int R_fgetc(FILE *stream) { return fgetc(stream); }

/* copy of R_ExpandFileName_unix */
static int HaveHOME=-1;
static char UserHOME[PATH_MAX];
const char *R_ExpandFileName_readline(const char *s, char *buff)
{
    char *p;

    if(s[0] != '~') return s;
    if(strlen(s) > 1 && s[1] != '/') return s;
    if(HaveHOME < 0) {
	p = getenv("HOME");
	if(p && *p && (strlen(p) < PATH_MAX)) {
	    strcpy(UserHOME, p);
	    HaveHOME = 1;
	} else
	    HaveHOME = 0;
    }
    if(HaveHOME > 0 && (strlen(UserHOME) + strlen(s+1) < PATH_MAX)) {
	strcpy(buff, UserHOME);
	strcat(buff, s+1);
	return buff;
    } else return s;
}

/* from sys-unix.c */
static char newFileName[PATH_MAX];

const char *R_ExpandFileName(const char *s)
{
    return R_ExpandFileName_readline(s, newFileName);
}

void R_set_command_line_arguments(int argc, char **argv) {}

void R_PolledEvents(void) {}

#ifndef RMIN_LOADFILE
SEXP R_LoadFromFile(FILE *fp, int n) { return R_NilValue; }

void R_RestoreGlobalEnvFromFile(const char *name, Rboolean qiet) {}

void R_SaveGlobalEnvToFile(const char *name) {}

void R_SaveToFileV(SEXP x, FILE *fp, int a, int b) {}
#endif

#ifdef RMIN_PROFILE
FILE *R_OpenInitFile(void) { return fopen(".Rprofile", "r"); }
#else
FILE *R_OpenInitFile(void) { return NULL; }
#endif

int Rconn_printf(Rconnection con, const char *format, ...)
{
    va_list argp;
    va_start(argp, format);
    int val = vprintf(format, argp);
    va_end(argp);
    return val;
}

int Rconn_fgetc(Rconnection con) { return fgetc(stdin); }

void Rstd_addhistory(SEXP call, SEXP op, SEXP args, SEXP env) {}
void Rstd_Busy(int which) {}
int Rstd_ChooseFile(int _new, char *buf, int len) { return 0; }
void Rstd_read_history(const char *s) {}
void Rstd_savehistory(SEXP call, SEXP op, SEXP args, SEXP env) {}
void Rstd_loadhistory(SEXP call, SEXP op, SEXP args, SEXP env) {}
void Rstd_CleanUp(SA_TYPE saveact, int status, int runLast) { exit(status); }
void Rstd_FlushConsole() { fflush(stdout); }
void Rstd_ResetConsole() { fflush(stdout); }
void Rstd_ClearerrConsole() { fflush(stderr); }
void R_ProcessEvents(void) {}


int  Rstd_ShowFiles(int nfile, const char **file, const char **headers,
		    const char *wtitle, Rboolean del, const char *pager)
{
    return 0;
}

void attribute_hidden Rstd_ShowMessage(const char *s)
{
    REprintf("%s\n", s);
}

void attribute_hidden Rstd_Suicide(const char *s)
{
    REprintf("Fatal error: %s\n", s);
    /* Might be called before translation is running */
    R_CleanUp(SA_SUICIDE, 2, 0);
}

void attribute_hidden Rstd_WriteConsole(const char *buf, int len)
{
    printf("%s", buf);
    fflush(stdout);
}

void attribute_hidden Rstd_WriteConsoleEx(const char *buf, int len, int otype)
{
    Rstd_WriteConsole(buf, len);
}

int attribute_hidden
Rstd_ReadConsole(const char *prompt, unsigned char *buf, int len,
		 int addtohistory)
{
    size_t ll;
    int err = 0;
    if (!R_Slave) {
	fputs(prompt, stdout);
	fflush(stdout); /* make sure prompt is output */
    }
    if (fgets((char *)buf, len, stdin) == NULL)
	return 0;
    ll = strlen((char *)buf);
    /* remove CR in CRLF ending */
    if (ll >= 2 && buf[ll - 1] == '\n' && buf[ll - 2] == '\r') {
	buf[ll - 2] = '\n';
	buf[--ll] = '\0';
    }
    /* according to system.txt, should be terminated in \n, so check this
       at eof and error */
    if ((err || feof(stdin))
	&& (ll == 0 || buf[ll - 1] != '\n') && ll < len) {
	buf[ll++] = '\n'; buf[ll] = '\0';
    }
    return 1;
}

/* from sys-unix.c */
#ifdef __FreeBSD__
# ifdef HAVE_FLOATINGPOINT_H
#  include <floatingpoint.h>
# endif
#endif

#ifdef linux
# ifdef HAVE_FPU_CONTROL_H
#  include <fpu_control.h>
# endif
#endif

/* patch from Ei-ji Nakama for Intel compilers on ix86.
   From http://www.nakama.ne.jp/memo/ia32_linux/R-2.1.1.iccftzdaz.patch.txt.
   Since updated to include x86_64.
 */
#if (defined(__i386) || defined(__x86_64)) && defined(__INTEL_COMPILER) && __INT
EL_COMPILER > 800
#include <xmmintrin.h>
#include <pmmintrin.h>
#endif

void fpu_setup(Rboolean start)
{
    if (start) {
#ifdef __FreeBSD__
    fpsetmask(0);
#endif

#ifdef NEED___SETFPUCW
    __setfpucw(_FPU_IEEE);
#endif
#if (defined(__i386) || defined(__x86_64)) && defined(__INTEL_COMPILER) && __INT
EL_COMPILER > 800
    _MM_SET_FLUSH_ZERO_MODE(_MM_FLUSH_ZERO_OFF);
    _MM_SET_DENORMALS_ZERO_MODE(_MM_DENORMALS_ZERO_OFF);
#endif
    } else {
#ifdef __FreeBSD__
    fpsetmask(~0);
#endif

#ifdef NEED___SETFPUCW
    __setfpucw(_FPU_DEFAULT);
#endif
    }
}

/* from sys-unix.c */
static double clk_tck, StartTime;
void R_setStartTime(void)
{
#ifdef HAVE_SYSCONF
    clk_tck = (double) sysconf(_SC_CLK_TCK);
#else
# ifndef CLK_TCK
/* this is in ticks/second, generally 60 on BSD style Unix, 100? on SysV
 */
#  ifdef HZ
#   define CLK_TCK HZ
#  else
#   define CLK_TCK 60
#  endif
# endif /* not CLK_TCK */
    clk_tck = (double) CLK_TCK;
#endif
    /* printf("CLK_TCK = %d\n", CLK_TCK); */
    StartTime = currentTime();
}

/* from sys-unix.c */
#ifdef HAVE_UNISTD_H
# include <unistd.h>
#endif

#ifndef HAVE_GETRUSAGE
# ifdef HAVE_SYS_TIME_H
#  include <sys/times.h>
# endif
#endif

#if defined(HAVE_SYS_RESOURCE_H) && defined(HAVE_GETRUSAGE)
/* on MacOS X it seems sys/resource.h needs sys/time.h first */
# include <sys/time.h>
# include <sys/resource.h>
#endif

void R_getProcTime(double *data)
{
    /* docs say this is rounded to the nearest ms */
    double et = currentTime() - StartTime;
    data[2] = 1e-3 * rint(1000*et);
#ifdef HAVE_GETRUSAGE
    /* all known current OSes */
    struct rusage self, children;
    getrusage(RUSAGE_SELF, &self);
    getrusage(RUSAGE_CHILDREN, &children);
    data[0] = (double) self.ru_utime.tv_sec +
	1e-3 * (self.ru_utime.tv_usec/1000);
    data[1] = (double) self.ru_stime.tv_sec +
	1e-3 * (self.ru_stime.tv_usec/1000);
    data[3] = (double) children.ru_utime.tv_sec +
	1e-3 * (children.ru_utime.tv_usec/1000);
    data[4] = (double) children.ru_stime.tv_sec +
	1e-3 * (children.ru_stime.tv_usec/1000);
#else
    /* Not known to be currently used */
    struct tms timeinfo;
    times(&timeinfo);
    data[0] = fround(timeinfo.tms_utime / clk_tck, 3);
    data[1] = fround(timeinfo.tms_stime / clk_tck, 3);
    data[3] = fround(timeinfo.tms_cutime / clk_tck, 3);
    data[4] = fround(timeinfo.tms_cstime / clk_tck, 3);
#endif
}

/* from fround.c */
#include "nmath.h"

/*  nearbyint is C99, so all platforms should have it (and AFAIK, all do) */
#ifdef HAVE_NEARBYINT
# define R_rint nearbyint
#elif defined(HAVE_RINT)
# define R_rint rint
#else
# define R_rint private_rint
extern double private_rint(double x);

/* also used potentially in fprec.c and main/format.c */
double attribute_hidden private_rint(double x)
{
    double tmp, sgn = 1.0;
    long ltmp;

    if (x != x) return x;			/* NaN */

    if (x < 0.0) {
	x = -x;
	sgn = -1.0;
    }

    if(x < (double) LONG_MAX) { /* in <limits.h> is architecture dependent */
	ltmp = x + 0.5;
	/* implement round to even */
	if(fabs(x + 0.5 - ltmp) < 10*DBL_EPSILON
	   && (ltmp % 2 == 1)) ltmp--;
	tmp = ltmp;
    } else {
	/* ignore round to even: too small a point to bother */
	tmp = floor(x + 0.5);
    }
    return sgn * tmp;
}
#endif

double fround(double x, double digits) {
#define MAX_DIGITS DBL_MAX_10_EXP
    /* = 308 (IEEE); was till R 0.99: (DBL_DIG - 1) */
    /* Note that large digits make sense for very small numbers */
    LDOUBLE pow10, sgn, intx;
    int dig;

    if (ISNAN(x) || ISNAN(digits))
	return x + digits;
    if(!R_FINITE(x)) return x;

    if(digits == ML_POSINF) return x;
    else if(digits == ML_NEGINF) return 0.0;

    if (digits > MAX_DIGITS) digits = MAX_DIGITS;
    dig = (int)floor(digits + 0.5);
    if(x < 0.) {
	sgn = -1.;
	x = -x;
    } else
	sgn = 1.;
    if (dig == 0) {
	return (double)(sgn * R_rint(x));
    } else if (dig > 0) {
        pow10 = R_pow_di(10., dig);
	intx = floor(x);
	return (double)(sgn * (intx + R_rint((double)((x-intx) * pow10)) / pow10));
    } else {
        pow10 = R_pow_di(10., -dig);
        return (double)(sgn * R_rint((double)(x/pow10)) * pow10);
    }
}

#ifndef RMIN_OPTIONS
/* options.c*/
SEXP GetOption(SEXP tag, SEXP rho) { return R_NilValue; }
void InitOptions(void) {}
int GetOptionCutoff(void) { return 60; }
int GetOptionDigits(void) { return 7; }
int GetOptionWidth(void) { return 80; }

SEXP GetOption1(SEXP tag)
{
    if (TYPEOF(tag) != SYMSXP) error("bad option selector");
    else if (! strcmp(CHAR(PRINTNAME(tag)), "keep.source")) return mkFalse();
    else if (! strcmp(CHAR(PRINTNAME(tag)), "prompt")) return mkString("> ");
    else if (! strcmp(CHAR(PRINTNAME(tag)), "continue")) return mkString("+ ");
    else if (! strcmp(CHAR(PRINTNAME(tag)), "check.bounds")) return mkFalse();
    else return R_NilValue;
}
#endif

#ifndef RMIN_SORT
/* sort.c */
void sortVector(SEXP s, Rboolean decreasing) {}
void orderVector1(int *indx, int n, SEXP key, Rboolean nalast,
		  Rboolean decreasing, SEXP rho)
{
}
#endif

#define NOTSUP(form) SEXP form { error("not supported"); return R_NilValue; }

#ifndef RMIN_SUBSTUFF
/* subset.c */
NOTSUP(R_subset3_dflt(SEXP x, SEXP input, SEXP call))

/* subassign.c */
NOTSUP(R_subassign3_dflt(SEXP call, SEXP x, SEXP nlist, SEXP val))
#endif

#ifndef RMIN_ARRAY_BIND
/* array.c */
NOTSUP(allocArray(SEXPTYPE mode, SEXP dims))
#endif

/* unique.c */
#ifndef RMIN_SUBSTUFF
int any_duplicated(SEXP x, Rboolean from_last) { return 0; }
#endif

#ifndef RMIN_ARRAY_BIND
/* bind.c */
SEXP attribute_hidden ItemName(SEXP names, R_xlen_t i)
{
  /* return  names[i]  if it is a character (>= 1 char), or NULL otherwise */
    if (names != R_NilValue &&
	STRING_ELT(names, i) != R_NilValue &&
	CHAR(STRING_ELT(names, i))[0] != '\0') /* length test */
	return STRING_ELT(names, i);
    else
	return R_NilValue;
}
#endif

#ifndef RMIN_BUILTIN
/* builtin.c */
NOTSUP(xlengthgets(SEXP x, R_xlen_t len))
NOTSUP(lengthgets(SEXP x, R_len_t len))
#endif

#ifndef RMIN_OBJECTS
/* objects.c */
Rboolean isMethodsDispatchOn(void) { return FALSE; }
int isBasicClass(const char *ss) { return FALSE; }
Rboolean isS4(SEXP s) { return FALSE; }
Rboolean R_has_methods(SEXP op) { return FALSE; }
NOTSUP(asS4(SEXP s, Rboolean flag, int complete))
NOTSUP(R_LookupMethod(SEXP method, SEXP rho, SEXP callrho, SEXP defrho))
NOTSUP(R_possible_dispatch(SEXP call, SEXP op, SEXP args, SEXP rho,
			   Rboolean promisedArgs))

int usemethod(const char *generic, SEXP obj, SEXP call, SEXP args,
	      SEXP rho, SEXP callrho, SEXP defrho, SEXP *ans)
{
    error("'usemethod' is not supported");
    return 0;
}
#endif


/* printarray.c */
void Rf_printMatrix(SEXP x, int offset, SEXP dim, int quote, int right,
		 SEXP rl, SEXP cl, const char *rn, const char *cn)
{
    error("not supported");
}

void Rf_printArray(SEXP x, SEXP dim, int quote, int right, SEXP dimnames)
{
    error("not supported");
}
  
/* complex.c */
#ifndef RMIN_COMPLEX
void attribute_hidden z_prec_r(Rcomplex *r, Rcomplex *x, double digits)
{
  error("not supported");
}

NOTSUP(complex_binary(ARITHOP_TYPE code, SEXP s1, SEXP s2))
NOTSUP(complex_unary(ARITHOP_TYPE code, SEXP s1, SEXP call))
NOTSUP(complex_math1(SEXP call, SEXP op, SEXP args, SEXP env))
NOTSUP(complex_math2(SEXP call, SEXP op, SEXP args, SEXP env))
#endif

/* pretty.c */
double R_pretty(double *lo, double *up, int *ndiv, int min_n,
		double shrink_sml, double high_u_fact[],
	       int eps_correction, int return_bounds)
{
    error("not supported");
}

void R_max_col(double *matrix, int *nr, int *nc, int *maxes, int *ties_meth)
{
    error("not supported");
}

void
F77_NAME(dtrsm)(const char *side, const char *uplo,
		const char *transa, const char *diag,
		const int *m, const int *n, const double *alpha,
		const double *a, const int *lda,
		double *b, const int *ldb)
{
    error("not supported");
}

void
F77_NAME(dgemm)(const char *transa, const char *transb, const int *m,
		const int *n, const int *k, const double *alpha,
		const double *a, const int *lda,
		const double *b, const int *ldb,
		const double *beta, double *c, const int *ldc)
{
    error("not supported");
}

void
F77_NAME(zgemm)(const char *transa, const char *transb, const int *m,
		const int *n, const int *k, const Rcomplex *alpha,
		const Rcomplex *a, const int *lda,
		const Rcomplex *b, const int *ldb,
		const Rcomplex *beta, Rcomplex *c, const int *ldc)
{
    error("not supported");
}

void
F77_NAME(dsyrk)(const char *uplo, const char *trans,
		const int *n, const int *k,
		const double *alpha, const double *a, const int *lda,
		const double *beta, double *c, const int *ldc)
{
    error("not supported");
}

#include <R_ext/GraphicsEngine.h> /* GEDevDesc, GEgetDevice */
pGEDevDesc GEgetDevice(int i) { return NULL; }


/* from arithmetic.c */
#include <Rmath.h>
#define NOTSUPD(form) double form { error("not supported"); return 0.0; }
NOTSUPD(ftrunc(double x))
NOTSUPD(ptukey(double x, double a, double b, double c, int d, int e))
NOTSUPD(qtukey(double x, double a, double b, double c, int d, int e))
NOTSUPD(dhyper(double x, double a, double b, double c, int d))
NOTSUPD(phyper(double x, double a, double b, double c, int d, int e))
NOTSUPD(qhyper(double x, double a, double b, double c, int d, int e))
NOTSUPD(dnbeta(double x, double a, double b, double c, int d))
NOTSUPD(qnbeta(double x, double a, double b, double c, int d, int e))
NOTSUPD(pnbeta(double x, double a, double b, double c, int d, int e))
NOTSUPD(dnf(double x, double a, double b, double c, int d))
NOTSUPD(pnf(double x, double a, double b, double c, int d, int e))
NOTSUPD(qnf(double x, double a, double b, double c, int d, int e))

NOTSUPD(lgammafn(double x))
NOTSUPD(gammafn(double x))
NOTSUPD(digamma(double x))
NOTSUPD(trigamma(double x))

NOTSUPD(dnorm(double x, double a, double c, int d))
NOTSUPD(pnorm(double x, double a, double c, int d, int e))
NOTSUPD(qnorm(double x, double a, double c, int d, int e))
NOTSUPD(dgamma(double x, double a, double c, int d))
NOTSUPD(pgamma(double x, double a, double c, int d, int e))
NOTSUPD(qgamma(double x, double a, double c, int d, int e))
NOTSUPD(dlogis(double x, double a, double c, int d))
NOTSUPD(plogis(double x, double a, double c, int d, int e))
NOTSUPD(qlogis(double x, double a, double c, int d, int e))
NOTSUPD(dcauchy(double x, double a, double c, int d))
NOTSUPD(pcauchy(double x, double a, double c, int d, int e))
NOTSUPD(qcauchy(double x, double a, double c, int d, int e))
NOTSUPD(dbeta(double x, double a, double c, int d))
NOTSUPD(pbeta(double x, double a, double c, int d, int e))
NOTSUPD(qbeta(double x, double a, double c, int d, int e))
NOTSUPD(dbinom(double x, double a, double c, int d))
NOTSUPD(pbinom(double x, double a, double c, int d, int e))
NOTSUPD(qbinom(double x, double a, double c, int d, int e))
NOTSUPD(dunif(double x, double a, double c, int d))
NOTSUPD(punif(double x, double a, double c, int d, int e))
NOTSUPD(qunif(double x, double a, double c, int d, int e))
NOTSUPD(dlnorm(double x, double a, double c, int d))
NOTSUPD(plnorm(double x, double a, double c, int d, int e))
NOTSUPD(qlnorm(double x, double a, double c, int d, int e))
NOTSUPD(dwilcox(double x, double a, double c, int d))
NOTSUPD(pwilcox(double x, double a, double c, int d, int e))
NOTSUPD(qwilcox(double x, double a, double c, int d, int e))
NOTSUPD(dweibull(double x, double a, double c, int d))
NOTSUPD(pweibull(double x, double a, double c, int d, int e))
NOTSUPD(qweibull(double x, double a, double c, int d, int e))
NOTSUPD(dnbinom(double x, double a, double c, int d))
NOTSUPD(pnbinom(double x, double a, double c, int d, int e))
NOTSUPD(qnbinom(double x, double a, double c, int d, int e))
NOTSUPD(df(double x, double a, double c, int d))
NOTSUPD(pf(double x, double a, double c, int d, int e))
NOTSUPD(qf(double x, double a, double c, int d, int e))
NOTSUPD(dnt(double x, double a, double c, int d))
NOTSUPD(pnt(double x, double a, double c, int d, int e))
NOTSUPD(qnt(double x, double a, double c, int d, int e))
NOTSUPD(dnchisq(double x, double a, double c, int d))
NOTSUPD(pnchisq(double x, double a, double c, int d, int e))
NOTSUPD(qnchisq(double x, double a, double c, int d, int e))
NOTSUPD(dnbinom_mu(double x, double a, double c, int d))
NOTSUPD(pnbinom_mu(double x, double a, double c, int d, int e))
NOTSUPD(qnbinom_mu(double x, double a, double c, int d, int e))
NOTSUPD(bessel_i_ex(double x, double a, double c, double *p))
NOTSUPD(bessel_k_ex(double x, double a, double c, double *p))

NOTSUPD(dpois(double x, double a, int d))
NOTSUPD(ppois(double x, double a, int d, int e))
NOTSUPD(qpois(double x, double a, int d, int e))
NOTSUPD(dsignrank(double x, double a, int d))
NOTSUPD(psignrank(double x, double a, int d, int e))
NOTSUPD(qsignrank(double x, double a, int d, int e))
NOTSUPD(dexp(double x, double a, int d))
NOTSUPD(pexp(double x, double a, int d, int e))
NOTSUPD(qexp(double x, double a, int d, int e))
NOTSUPD(dt(double x, double a, int d))
NOTSUPD(pt(double x, double a, int d, int e))
NOTSUPD(qt(double x, double a, int d, int e))
NOTSUPD(dgeom(double x, double a, int d))
NOTSUPD(beta(double x, double a))
NOTSUPD(lbeta(double x, double a))
NOTSUPD(choose(double x, double a))
NOTSUPD(lchoose(double x, double a))
NOTSUPD(psigamma(double x, double a))
NOTSUPD(fprec(double x, double a))
NOTSUPD(bessel_j_ex(double x, double a, double *p))
NOTSUPD(bessel_y_ex(double x, double a, double *p))

NOTSUPD(pgeom(double x, double a, int d, int e))
NOTSUPD(qgeom(double x, double a, int d, int e))
NOTSUPD(dchisq(double x, double a, int d))
NOTSUPD(pchisq(double x, double a, int d, int e))
NOTSUPD(qchisq(double x, double a, int d, int e))


/* names.c */
#define NOTSUPBLTIN(name) NOTSUP(name(SEXP call, SEXP op, SEXP args, SEXP rho))

NOTSUPBLTIN(do_adist)
NOTSUPBLTIN(do_agrep)
NOTSUPBLTIN(do_allnames)
NOTSUPBLTIN(do_aregexec)
NOTSUPBLTIN(do_asPOSIXct)
NOTSUPBLTIN(do_asPOSIXlt)
NOTSUPBLTIN(do_capabilities)
NOTSUPBLTIN(do_capabilitiesX11)
NOTSUPBLTIN(do_charToRaw)
NOTSUPBLTIN(do_clearpushback)
NOTSUPBLTIN(do_close)
NOTSUPBLTIN(do_commandArgs)
NOTSUPBLTIN(do_Cstack_info)
NOTSUPBLTIN(do_cum)
NOTSUPBLTIN(do_D2POSIXlt)
NOTSUPBLTIN(do_date)
NOTSUPBLTIN(do_debug)
NOTSUPBLTIN(do_dircreate)
NOTSUPBLTIN(do_dotcall)
NOTSUPBLTIN(do_dotcallgr)
NOTSUPBLTIN(do_dotCode)
NOTSUPBLTIN(do_dynload)
NOTSUPBLTIN(do_dynunload)
NOTSUPBLTIN(do_External)
NOTSUPBLTIN(do_Externalgr)
NOTSUPBLTIN(do_fifo)
NOTSUPBLTIN(do_fileaccess)
NOTSUPBLTIN(do_fileappend)
NOTSUPBLTIN(do_filechoose)
NOTSUPBLTIN(do_filecopy)
NOTSUPBLTIN(do_filecreate)
NOTSUPBLTIN(do_fileexists)
NOTSUPBLTIN(do_fileinfo)
NOTSUPBLTIN(do_filelink)
NOTSUPBLTIN(do_filepath)
NOTSUPBLTIN(do_fileremove)
NOTSUPBLTIN(do_filerename)
NOTSUPBLTIN(do_fileshow)
NOTSUPBLTIN(do_filesymlink)
NOTSUPBLTIN(do_first_min)
NOTSUPBLTIN(do_flush)
NOTSUPBLTIN(do_format)
NOTSUPBLTIN(do_formatinfo)
NOTSUPBLTIN(do_formatPOSIXlt)
NOTSUPBLTIN(do_getallconnections)
NOTSUPBLTIN(do_getconnection)
NOTSUPBLTIN(do_getDllTable)
NOTSUPBLTIN(do_getlocale)
NOTSUPBLTIN(do_getRegisteredRoutines)
NOTSUPBLTIN(do_getSymbolInfo)
NOTSUPBLTIN(do_grep)
NOTSUPBLTIN(do_grepraw)
NOTSUPBLTIN(do_gsub)
NOTSUPBLTIN(do_gzcon)
NOTSUPBLTIN(do_gzfile)
NOTSUPBLTIN(do_inspect)
NOTSUPBLTIN(do_intToBits)
NOTSUPBLTIN(do_intToUtf8)
NOTSUPBLTIN(do_isatty)
NOTSUPBLTIN(do_isincomplete)
NOTSUPBLTIN(do_islistfactor)
NOTSUPBLTIN(do_isloaded)
NOTSUPBLTIN(do_isopen)
NOTSUPBLTIN(do_isseekable)
NOTSUPBLTIN(do_l10n_info)
NOTSUPBLTIN(do_lapack)
NOTSUPBLTIN(do_lapply)
NOTSUPBLTIN(do_listdirs)
NOTSUPBLTIN(do_listfiles)
NOTSUPBLTIN(do_localeconv)
NOTSUPBLTIN(do_machine)
NOTSUPBLTIN(do_mapply)
NOTSUPBLTIN(do_memCompress)
NOTSUPBLTIN(do_memDecompress)
NOTSUPBLTIN(do_open)
NOTSUPBLTIN(do_packBits)
NOTSUPBLTIN(do_parse)
NOTSUPBLTIN(do_paste)
NOTSUPBLTIN(do_pathexpand)
NOTSUPBLTIN(do_pipe)
NOTSUPBLTIN(do_pmin)
NOTSUPBLTIN(do_POSIXlt2D)
NOTSUPBLTIN(do_pushback)
NOTSUPBLTIN(do_pushbacklength)
NOTSUPBLTIN(do_qsort)
NOTSUPBLTIN(do_random1)
NOTSUPBLTIN(do_random2)
NOTSUPBLTIN(do_random3)
NOTSUPBLTIN(do_range)
NOTSUPBLTIN(do_rapply)
NOTSUPBLTIN(do_rawconnection)
NOTSUPBLTIN(do_rawconvalue)
NOTSUPBLTIN(do_rawShift)
NOTSUPBLTIN(do_rawToBits)
NOTSUPBLTIN(do_rawToChar)
NOTSUPBLTIN(do_readbin)
NOTSUPBLTIN(do_readchar)
NOTSUPBLTIN(do_readDCF)
NOTSUPBLTIN(do_readEnviron)
NOTSUPBLTIN(do_readLines)
NOTSUPBLTIN(do_readlink)
NOTSUPBLTIN(do_readln)
NOTSUPBLTIN(do_recordGraphics)
NOTSUPBLTIN(do_regexec)
NOTSUPBLTIN(do_regexpr)
NOTSUPBLTIN(do_retracemem)
NOTSUPBLTIN(do_Rhome)
NOTSUPBLTIN(do_sample)
NOTSUPBLTIN(do_scan)
NOTSUPBLTIN(do_seek)
NOTSUPBLTIN(do_setFileTime)
NOTSUPBLTIN(do_setlocale)
NOTSUPBLTIN(do_sink)
NOTSUPBLTIN(do_sinknumber)
NOTSUPBLTIN(do_sockconn)
NOTSUPBLTIN(do_sockselect)
NOTSUPBLTIN(do_split)
NOTSUPBLTIN(do_sprintf)
NOTSUPBLTIN(do_stderr)
NOTSUPBLTIN(do_stdin)
NOTSUPBLTIN(do_stdout)
NOTSUPBLTIN(do_strptime)
NOTSUPBLTIN(do_strsplit)
NOTSUPBLTIN(do_sumconnection)
NOTSUPBLTIN(do_summary)
NOTSUPBLTIN(do_syschmod)
NOTSUPBLTIN(do_sysgetpid)
NOTSUPBLTIN(do_sysinfo)
NOTSUPBLTIN(do_syssleep)
NOTSUPBLTIN(do_system)
NOTSUPBLTIN(do_systime)
NOTSUPBLTIN(do_sysumask)
NOTSUPBLTIN(do_textconnection)
NOTSUPBLTIN(do_textconvalue)
NOTSUPBLTIN(do_trace)
NOTSUPBLTIN(do_tracemem)
NOTSUPBLTIN(do_traceOnOff)
NOTSUPBLTIN(do_truncate)
NOTSUPBLTIN(do_unlink)
NOTSUPBLTIN(do_untracemem)
NOTSUPBLTIN(do_unz)
NOTSUPBLTIN(do_url)
NOTSUPBLTIN(do_utf8ToInt)
NOTSUPBLTIN(do_vapply)
NOTSUPBLTIN(do_version)
NOTSUPBLTIN(do_which)
NOTSUPBLTIN(do_writebin)
NOTSUPBLTIN(do_writechar)
NOTSUPBLTIN(do_writelines)

#ifndef RMIN_OPTIONS
NOTSUPBLTIN(do_options)
#endif

#ifndef RMIN_OBJECTS
NOTSUPBLTIN(do_S4on)
NOTSUPBLTIN(do_inherits)
NOTSUPBLTIN(do_nextmethod)
NOTSUPBLTIN(do_setS4Object)
NOTSUPBLTIN(do_standardGeneric)
NOTSUPBLTIN(do_unclass)
NOTSUPBLTIN(do_usemethod)
#endif

#ifndef RMIN_BUILTIN
NOTSUPBLTIN(do_args)
NOTSUPBLTIN(do_body)
NOTSUPBLTIN(do_bodyCode)
NOTSUPBLTIN(do_cat)
NOTSUPBLTIN(do_delayed)
NOTSUPBLTIN(do_envir)
NOTSUPBLTIN(do_envirName)
NOTSUPBLTIN(do_envirgets)
NOTSUPBLTIN(do_expression)
NOTSUPBLTIN(do_formals)
NOTSUPBLTIN(do_lengthgets)
NOTSUPBLTIN(do_makelazy)
NOTSUPBLTIN(do_makelist)
NOTSUPBLTIN(do_makevector)
NOTSUPBLTIN(do_newenv)
NOTSUPBLTIN(do_onexit)
NOTSUPBLTIN(do_parentenv)
NOTSUPBLTIN(do_parentenvgets)
NOTSUPBLTIN(do_switch)
#endif

#ifndef RMIN_SORT
NOTSUPBLTIN(do_isunsorted)
NOTSUPBLTIN(do_order)
NOTSUPBLTIN(do_psort)
NOTSUPBLTIN(do_radixsort)
NOTSUPBLTIN(do_rank)
NOTSUPBLTIN(do_sort)
NOTSUPBLTIN(do_xtfrm)
#endif

#ifndef RMIN_SEQ
NOTSUPBLTIN(do_rep)
NOTSUPBLTIN(do_rep_int)
NOTSUPBLTIN(do_rep_len)
NOTSUPBLTIN(do_seq)
NOTSUPBLTIN(do_seq_along)
NOTSUPBLTIN(do_seq_len)
NOTSUPBLTIN(do_colon)
#endif

#ifndef RMIN_CHARACTER
NOTSUPBLTIN(do_abbrev)
NOTSUPBLTIN(do_chartr)
NOTSUPBLTIN(do_makenames)
NOTSUPBLTIN(do_nchar)
NOTSUPBLTIN(do_nzchar)
NOTSUPBLTIN(do_strtoi)
NOTSUPBLTIN(do_strtrim)
NOTSUPBLTIN(do_substr)
NOTSUPBLTIN(do_substrgets)
NOTSUPBLTIN(do_tolower)
#endif

#ifndef RMIN_COMPLEX
NOTSUPBLTIN(do_cmathfuns)
NOTSUPBLTIN(do_complex)
NOTSUPBLTIN(do_polyroot)
#endif

#ifndef RMIN_LOGIC
NOTSUPBLTIN(do_logic)
NOTSUPBLTIN(do_logic2)
NOTSUPBLTIN(do_logic3)
#endif

#ifndef RMIN_RELOP
NOTSUPBLTIN(do_bitwise)
NOTSUPBLTIN(do_relop)
NOTSUPBLTIN(do_relop_dflt)
#endif

#ifndef RMIN_ARRAY_BIND
NOTSUPBLTIN(do_aperm)
NOTSUPBLTIN(do_array)
NOTSUPBLTIN(do_backsolve)
NOTSUPBLTIN(do_bind)
NOTSUPBLTIN(do_c)
NOTSUPBLTIN(do_c_dflt)
NOTSUPBLTIN(do_colsum)
NOTSUPBLTIN(do_diag)
NOTSUPBLTIN(do_drop)
NOTSUPBLTIN(do_length)
NOTSUPBLTIN(do_matprod)
NOTSUPBLTIN(do_matrix)
NOTSUPBLTIN(do_maxcol)
NOTSUPBLTIN(do_rowscols)
NOTSUPBLTIN(do_transpose)
NOTSUPBLTIN(do_unlist)
#endif

#ifndef RMIN_SUBSTUFF
NOTSUPBLTIN(do_match)
NOTSUPBLTIN(do_pmatch)
NOTSUPBLTIN(do_charmatch)
NOTSUPBLTIN(do_matchcall)
NOTSUPBLTIN(do_sample2)
NOTSUPBLTIN(do_rowsum)
NOTSUPBLTIN(do_makeunique)
NOTSUPBLTIN(do_duplicated)
NOTSUPBLTIN(do_subassign)
NOTSUPBLTIN(do_subassign2)
NOTSUPBLTIN(do_subassign2_dflt)
NOTSUPBLTIN(do_subassign3)
NOTSUPBLTIN(do_subassign_dflt)
NOTSUPBLTIN(do_subset)
NOTSUPBLTIN(do_subset2)
NOTSUPBLTIN(do_subset2_dflt)
NOTSUPBLTIN(do_subset3)
NOTSUPBLTIN(do_subset_dflt)
NOTSUPBLTIN(do_RNGkind)
NOTSUPBLTIN(do_setseed)
#endif

#ifndef RMIN_LOADFILE
NOTSUPBLTIN(do_getVarsFromFrame)
NOTSUPBLTIN(do_lazyLoadDBfetch)
NOTSUPBLTIN(do_lazyLoadDBflush)
NOTSUPBLTIN(do_lazyLoadDBinsertValue)
NOTSUPBLTIN(do_load)
NOTSUPBLTIN(do_loadFromConn2)
NOTSUPBLTIN(do_save)
NOTSUPBLTIN(do_saveToConn)
NOTSUPBLTIN(do_serialize)
NOTSUPBLTIN(do_serializeToConn)
NOTSUPBLTIN(do_unserializeFromConn)
#endif

#ifdef RMIN_LOADFILE
int Rconn_getline(Rconnection con, char *buf, int bufsize)
{
    error("not supported");
    return 0;
}

size_t R_WriteConnection(Rconnection con, void *buf, size_t n)
{
    error("not supported");
    return 0;
}

NOTSUP(R_decompress1(SEXP in, Rboolean *err))
NOTSUP(R_decompress2(SEXP in, Rboolean *err))
NOTSUP(R_decompress3(SEXP in, Rboolean *err))
NOTSUP(R_compress1(SEXP in))
NOTSUP(R_compress2(SEXP in))
NOTSUP(R_compress3(SEXP in))
#endif

/* from Rmain.c */
int Rf_initialize_R(int ac, char **av); /* in ../unix/system.c */

#include <Rinterface.h>
#include <stdlib.h>

extern int R_running_as_main_program;   /* in ../unix/system.c */

int main(int ac, char **av)
{
    setenv("R_HOME", "FOO", TRUE);
    R_running_as_main_program = 1;
    Rf_initialize_R(ac, av);
    Rf_mainloop(); /* does not return */
    return 0;
}

	/* Declarations to keep f77 happy */

int MAIN_(int ac, char **av)  {return 0;}
int MAIN__(int ac, char **av) {return 0;}
int __main(int ac, char **av) {return 0;}
