## Accept srcdir and builddir as positional command line arguments.
args <- commandArgs(trailingOnly = TRUE)
srcdir <- args[1]
builddir <- args[2]

## Identify the OS flavor. For now, only Ubuntu, Fedora, and MacOS are
## supported.
info <- Sys.info()["version"]
version <- info["version"]
OS <- {
    if (grepl("Ubuntu", version)) "Ubuntu"
    else if (grepl("Darwin", version)) "MacOS"
    else if (grepl("fc[0-1][0-9]", Sys.info()["release"])) "Fedora"
    else stop("can't identify the OS")
}

## The build directory is assumed to be R.home()
if (is.na(builddir)) builddir <- R.home()

## Find the source directory from the strings embedded in the executable
if (is.na(srcdir)) {
    if (OS == "MacOS")
        stop("for now the build directory has to be specified as an argument")
    strings <- system(sprintf("strings %s/bin/exec/R", builddir), intern = TRUE)
    idx <- which(grepl("main/main\\.c", strings))
    if (length(idx) > 0) {
        srcdir <- dirname(dirname(strings[idx][1]))
    } else stop("can't find R source directory")

    ## If the source directory is a relative path assume it is relative to
    ## the build directory.
    if (grepl("^..", srcdir))
        srcdir <- normalizePath(paste(builddir, "bin/exec", srcdir, sep = "/"))
}

## Compute the appropriate LIBFLAGS and INCFLAGS for the OS
libflags <- {
    if (OS == "MacOS")
        "-liconv $(BUILDDIR)/src/extra/xz/liblzma.a $(BUILDDIR)/src/extra/intl/libintl.a -Wl,-framework -Wl,CoreFoundation -licucore"
    else if (OS == "Ubuntu") "-lrt"
    else "-lrt -licuuc -licui18n"
}

incflags <- if (OS == "MacOS") "-I$(SRCDIR)/extra/xz/api" else ""

## Create and run the sed commands.
buildcmd <- sprintf("sed 's|@BUILDDIR@|%s|'", builddir)
srccmd <- sprintf("sed 's|@SRCDIR@|%s|'", srcdir)
libcmd <- sprintf("sed 's|@LIBFLAGS@|%s|'", libflags)
inccmd <- sprintf("sed 's|@INCFLAGS@|%s|'", incflags)

sedcmd <- paste("cat Makefile.in", buildcmd, srccmd, libcmd, inccmd,
                sep = " | ")

system(paste(sedcmd, "> Makefile"))
